package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.lite.domain.MetaDataSet;
import cn.ibizlab.core.lite.service.IMetaDataSetService;
import cn.ibizlab.core.lite.filter.MetaDataSetSearchContext;
import cn.ibizlab.util.annotation.VersionCheck;

@Slf4j
@Api(tags = {"数据集" })
@RestController("api-metadataset")
@RequestMapping("")
public class MetaDataSetResource {

    @Autowired
    public IMetaDataSetService metadatasetService;

    @Autowired
    @Lazy
    public MetaDataSetMapping metadatasetMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Create-all')")
    @ApiOperation(value = "新建数据集", tags = {"数据集" },  notes = "新建数据集")
	@RequestMapping(method = RequestMethod.POST, value = "/metadatasets")
    public ResponseEntity<MetaDataSetDTO> create(@Validated @RequestBody MetaDataSetDTO metadatasetdto) {
        MetaDataSet domain = metadatasetMapping.toDomain(metadatasetdto);
		metadatasetService.create(domain);
        MetaDataSetDTO dto = metadatasetMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Create-all')")
    @ApiOperation(value = "批量新建数据集", tags = {"数据集" },  notes = "批量新建数据集")
	@RequestMapping(method = RequestMethod.POST, value = "/metadatasets/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<MetaDataSetDTO> metadatasetdtos) {
        metadatasetService.createBatch(metadatasetMapping.toDomain(metadatasetdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Update-all')")
    @ApiOperation(value = "更新数据集", tags = {"数据集" },  notes = "更新数据集")
	@RequestMapping(method = RequestMethod.PUT, value = "/metadatasets/{metadataset_id}")
    public ResponseEntity<MetaDataSetDTO> update(@PathVariable("metadataset_id") String metadataset_id, @RequestBody MetaDataSetDTO metadatasetdto) {
		MetaDataSet domain  = metadatasetMapping.toDomain(metadatasetdto);
        domain .setDatasetId(metadataset_id);
		metadatasetService.update(domain );
		MetaDataSetDTO dto = metadatasetMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Update-all')")
    @ApiOperation(value = "批量更新数据集", tags = {"数据集" },  notes = "批量更新数据集")
	@RequestMapping(method = RequestMethod.PUT, value = "/metadatasets/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<MetaDataSetDTO> metadatasetdtos) {
        metadatasetService.updateBatch(metadatasetMapping.toDomain(metadatasetdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Remove-all')")
    @ApiOperation(value = "删除数据集", tags = {"数据集" },  notes = "删除数据集")
	@RequestMapping(method = RequestMethod.DELETE, value = "/metadatasets/{metadataset_id}")
    public ResponseEntity<Boolean> remove(@PathVariable("metadataset_id") String metadataset_id) {
         return ResponseEntity.status(HttpStatus.OK).body(metadatasetService.remove(metadataset_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Remove-all')")
    @ApiOperation(value = "批量删除数据集", tags = {"数据集" },  notes = "批量删除数据集")
	@RequestMapping(method = RequestMethod.DELETE, value = "/metadatasets/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        metadatasetService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Get-all')")
    @ApiOperation(value = "获取数据集", tags = {"数据集" },  notes = "获取数据集")
	@RequestMapping(method = RequestMethod.GET, value = "/metadatasets/{metadataset_id}")
    public ResponseEntity<MetaDataSetDTO> get(@PathVariable("metadataset_id") String metadataset_id) {
        MetaDataSet domain = metadatasetService.get(metadataset_id);
        MetaDataSetDTO dto = metadatasetMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "获取数据集草稿", tags = {"数据集" },  notes = "获取数据集草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/metadatasets/getdraft")
    public ResponseEntity<MetaDataSetDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(metadatasetMapping.toDto(metadatasetService.getDraft(new MetaDataSet())));
    }

    @ApiOperation(value = "检查数据集", tags = {"数据集" },  notes = "检查数据集")
	@RequestMapping(method = RequestMethod.POST, value = "/metadatasets/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody MetaDataSetDTO metadatasetdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(metadatasetService.checkKey(metadatasetMapping.toDomain(metadatasetdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Save-all')")
    @ApiOperation(value = "保存数据集", tags = {"数据集" },  notes = "保存数据集")
	@RequestMapping(method = RequestMethod.POST, value = "/metadatasets/save")
    public ResponseEntity<Boolean> save(@RequestBody MetaDataSetDTO metadatasetdto) {
        return ResponseEntity.status(HttpStatus.OK).body(metadatasetService.save(metadatasetMapping.toDomain(metadatasetdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-Save-all')")
    @ApiOperation(value = "批量保存数据集", tags = {"数据集" },  notes = "批量保存数据集")
	@RequestMapping(method = RequestMethod.POST, value = "/metadatasets/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<MetaDataSetDTO> metadatasetdtos) {
        metadatasetService.saveBatch(metadatasetMapping.toDomain(metadatasetdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-searchDefault-all')")
	@ApiOperation(value = "获取数据集", tags = {"数据集" } ,notes = "获取数据集")
    @RequestMapping(method= RequestMethod.GET , value="/metadatasets/fetchdefault")
	public ResponseEntity<List<MetaDataSetDTO>> fetchDefault(MetaDataSetSearchContext context) {
        Page<MetaDataSet> domains = metadatasetService.searchDefault(context) ;
        List<MetaDataSetDTO> list = metadatasetMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaDataSet-searchDefault-all')")
	@ApiOperation(value = "查询数据集", tags = {"数据集" } ,notes = "查询数据集")
    @RequestMapping(method= RequestMethod.POST , value="/metadatasets/searchdefault")
	public ResponseEntity<Page<MetaDataSetDTO>> searchDefault(@RequestBody MetaDataSetSearchContext context) {
        Page<MetaDataSet> domains = metadatasetService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(metadatasetMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

