package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.lite.domain.MetaRelationship;
import cn.ibizlab.core.lite.service.IMetaRelationshipService;
import cn.ibizlab.core.lite.filter.MetaRelationshipSearchContext;
import cn.ibizlab.util.annotation.VersionCheck;

@Slf4j
@Api(tags = {"实体关系" })
@RestController("api-metarelationship")
@RequestMapping("")
public class MetaRelationshipResource {

    @Autowired
    public IMetaRelationshipService metarelationshipService;

    @Autowired
    @Lazy
    public MetaRelationshipMapping metarelationshipMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Create-all')")
    @ApiOperation(value = "新建实体关系", tags = {"实体关系" },  notes = "新建实体关系")
	@RequestMapping(method = RequestMethod.POST, value = "/metarelationships")
    public ResponseEntity<MetaRelationshipDTO> create(@Validated @RequestBody MetaRelationshipDTO metarelationshipdto) {
        MetaRelationship domain = metarelationshipMapping.toDomain(metarelationshipdto);
		metarelationshipService.create(domain);
        MetaRelationshipDTO dto = metarelationshipMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Create-all')")
    @ApiOperation(value = "批量新建实体关系", tags = {"实体关系" },  notes = "批量新建实体关系")
	@RequestMapping(method = RequestMethod.POST, value = "/metarelationships/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<MetaRelationshipDTO> metarelationshipdtos) {
        metarelationshipService.createBatch(metarelationshipMapping.toDomain(metarelationshipdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Update-all')")
    @ApiOperation(value = "更新实体关系", tags = {"实体关系" },  notes = "更新实体关系")
	@RequestMapping(method = RequestMethod.PUT, value = "/metarelationships/{metarelationship_id}")
    public ResponseEntity<MetaRelationshipDTO> update(@PathVariable("metarelationship_id") String metarelationship_id, @RequestBody MetaRelationshipDTO metarelationshipdto) {
		MetaRelationship domain  = metarelationshipMapping.toDomain(metarelationshipdto);
        domain .setId(metarelationship_id);
		metarelationshipService.update(domain );
		MetaRelationshipDTO dto = metarelationshipMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Update-all')")
    @ApiOperation(value = "批量更新实体关系", tags = {"实体关系" },  notes = "批量更新实体关系")
	@RequestMapping(method = RequestMethod.PUT, value = "/metarelationships/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<MetaRelationshipDTO> metarelationshipdtos) {
        metarelationshipService.updateBatch(metarelationshipMapping.toDomain(metarelationshipdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Remove-all')")
    @ApiOperation(value = "删除实体关系", tags = {"实体关系" },  notes = "删除实体关系")
	@RequestMapping(method = RequestMethod.DELETE, value = "/metarelationships/{metarelationship_id}")
    public ResponseEntity<Boolean> remove(@PathVariable("metarelationship_id") String metarelationship_id) {
         return ResponseEntity.status(HttpStatus.OK).body(metarelationshipService.remove(metarelationship_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Remove-all')")
    @ApiOperation(value = "批量删除实体关系", tags = {"实体关系" },  notes = "批量删除实体关系")
	@RequestMapping(method = RequestMethod.DELETE, value = "/metarelationships/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        metarelationshipService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Get-all')")
    @ApiOperation(value = "获取实体关系", tags = {"实体关系" },  notes = "获取实体关系")
	@RequestMapping(method = RequestMethod.GET, value = "/metarelationships/{metarelationship_id}")
    public ResponseEntity<MetaRelationshipDTO> get(@PathVariable("metarelationship_id") String metarelationship_id) {
        MetaRelationship domain = metarelationshipService.get(metarelationship_id);
        MetaRelationshipDTO dto = metarelationshipMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "获取实体关系草稿", tags = {"实体关系" },  notes = "获取实体关系草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/metarelationships/getdraft")
    public ResponseEntity<MetaRelationshipDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(metarelationshipMapping.toDto(metarelationshipService.getDraft(new MetaRelationship())));
    }

    @ApiOperation(value = "检查实体关系", tags = {"实体关系" },  notes = "检查实体关系")
	@RequestMapping(method = RequestMethod.POST, value = "/metarelationships/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody MetaRelationshipDTO metarelationshipdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(metarelationshipService.checkKey(metarelationshipMapping.toDomain(metarelationshipdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Save-all')")
    @ApiOperation(value = "保存实体关系", tags = {"实体关系" },  notes = "保存实体关系")
	@RequestMapping(method = RequestMethod.POST, value = "/metarelationships/save")
    public ResponseEntity<Boolean> save(@RequestBody MetaRelationshipDTO metarelationshipdto) {
        return ResponseEntity.status(HttpStatus.OK).body(metarelationshipService.save(metarelationshipMapping.toDomain(metarelationshipdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-Save-all')")
    @ApiOperation(value = "批量保存实体关系", tags = {"实体关系" },  notes = "批量保存实体关系")
	@RequestMapping(method = RequestMethod.POST, value = "/metarelationships/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<MetaRelationshipDTO> metarelationshipdtos) {
        metarelationshipService.saveBatch(metarelationshipMapping.toDomain(metarelationshipdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-searchDefault-all')")
	@ApiOperation(value = "获取数据集", tags = {"实体关系" } ,notes = "获取数据集")
    @RequestMapping(method= RequestMethod.GET , value="/metarelationships/fetchdefault")
	public ResponseEntity<List<MetaRelationshipDTO>> fetchDefault(MetaRelationshipSearchContext context) {
        Page<MetaRelationship> domains = metarelationshipService.searchDefault(context) ;
        List<MetaRelationshipDTO> list = metarelationshipMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzlite-MetaRelationship-searchDefault-all')")
	@ApiOperation(value = "查询数据集", tags = {"实体关系" } ,notes = "查询数据集")
    @RequestMapping(method= RequestMethod.POST , value="/metarelationships/searchdefault")
	public ResponseEntity<Page<MetaRelationshipDTO>> searchDefault(@RequestBody MetaRelationshipSearchContext context) {
        Page<MetaRelationship> domains = metarelationshipService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(metarelationshipMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

